@@ -5,11 +5,11 @@ from trytond.model import ModelView, ModelSQL, fields
 from trytond.wizard import Wizard
 from trytond.report import Report
 from trytond.tools import reduce_ids
+from trytond.pyson import Equal, Eval, Not, PYSONEncoder, Date
 from decimal import Decimal
 import datetime
 import time
 import os
-import mx.DateTime
 
 
 class TypeTemplate(ModelSQL, ModelView):
@@ -41,7 +41,7 @@ class TypeTemplate(ModelSQL, ModelView):
     def default_display_balance(self, cursor, user, context=None):
         return 'debit-credit'
 
-    def get_rec_name(self, cursor, user, ids, name, arg, context=None):
+    def get_rec_name(self, cursor, user, ids, name, context=None):
         if not ids:
             return {}
         res = {}
@@ -116,6 +116,10 @@ class TypeTemplate(ModelSQL, ModelView):
             new_id = type_obj.create(cursor, user, vals, context=context)
 
             prev_lang = template._context.get('language') or 'en_US'
+            prev_data = {}
+            for field_name, field in template._columns.iteritems():
+                if getattr(field, 'translate', False):
+                    prev_data[field_name] = template[field_name]
             ctx = context.copy()
             for lang in lang_obj.get_translatable_languages(cursor, user,
                     context=context):
@@ -124,9 +128,10 @@ class TypeTemplate(ModelSQL, ModelView):
                 ctx['language'] = lang
                 template.setLang(lang)
                 data = {}
-                for field in template._columns.keys():
-                    if template._columns[field].translate:
-                        data[field] = template[field]
+                for field_name, field in template._columns.iteritems():
+                    if getattr(field, 'translate', False) \
+                            and template[field_name] != prev_data[field_name]:
+                        data[field_name] = template[field_name]
                 if data:
                     type_obj.write(cursor, user, new_id, data, context=ctx)
             template.setLang(prev_lang)
@@ -153,10 +158,11 @@ class Type(ModelSQL, ModelView):
     childs = fields.One2Many('account.account.type', 'parent', 'Children')
     sequence = fields.Integer('Sequence', required=True,
             help='Use to order the account type')
-    currency_digits = fields.Function('get_currency_digits', type='integer',
-            string='Currency Digits')
-    amount = fields.Function('get_amount', digits="(16, currency_digits)",
-            string='Amount', depends=['currency_digits'])
+    currency_digits = fields.Function(fields.Integer('Currency Digits'),
+            'get_currency_digits')
+    amount = fields.Function(fields.Numeric('Amount',
+        digits=(16, Eval('currency_digits', 2)), depends=['currency_digits']),
+        'get_amount')
     balance_sheet = fields.Boolean('Balance Sheet')
     income_statement = fields.Boolean('Income Statement')
     display_balance = fields.Selection([
@@ -180,13 +186,13 @@ class Type(ModelSQL, ModelView):
     def default_display_balance(self, cursor, user, context=None):
         return 'debit-credit'
 
-    def get_currency_digits(self, cursor, user, ids, name, arg, context=None):
+    def get_currency_digits(self, cursor, user, ids, name, context=None):
         res = {}
         for type in self.browse(cursor, user, ids, context=context):
             res[type.id] = type.company.currency.digits
         return res
 
-    def get_amount(self, cursor, user, ids, name, arg, context=None):
+    def get_amount(self, cursor, user, ids, name, context=None):
         account_obj = self.pool.get('account.account')
         currency_obj = self.pool.get('currency.currency')
 
@@ -224,7 +230,7 @@ class Type(ModelSQL, ModelView):
                 res[type.id] = - res[type.id]
         return res
 
-    def get_rec_name(self, cursor, user, ids, name, arg, context=None):
+    def get_rec_name(self, cursor, user, ids, name, context=None):
         if not ids:
             return {}
         res = {}
@@ -273,6 +279,10 @@ class Type(ModelSQL, ModelView):
                 self.write(cursor, user, type.id, vals, context=context)
 
             prev_lang = type._context.get('language') or 'en_US'
+            prev_data = {}
+            for field_name, field in template._columns.iteritems():
+                if getattr(field, 'translate', False):
+                    prev_data[field_name] = template[field_name]
             ctx = context.copy()
             for lang in lang_obj.get_translatable_languages(cursor, user,
                     context=context):
@@ -280,8 +290,11 @@ class Type(ModelSQL, ModelView):
                     continue
                 ctx['language'] = lang
                 type.setLang(lang)
-                data = template_obj._get_type_value(cursor, user,
-                        type.template, context=ctx, type=type)
+                data = {}
+                for field_name, field in template._columns.iteritems():
+                    if getattr(field, 'translate', False) \
+                            and template[field_name] != prev_data[field_name]:
+                        data[field_name] = template[field_name]
                 if data:
                     self.write(cursor, user, type.id, data, context=ctx)
             type.setLang(prev_lang)
@@ -305,15 +318,15 @@ class AccountTemplate(ModelSQL, ModelView):
     type = fields.Many2One('account.account.type.template', 'Type',
             ondelete="RESTRICT",
             states={
-                'invisible': "kind == 'view'",
-                'required': "kind != 'view'",
+                'invisible': Equal(Eval('kind'), 'view'),
+                'required': Not(Equal(Eval('kind'), 'view')),
             }, depends=['kind'])
     parent = fields.Many2One('account.account.template', 'Parent', select=1,
             ondelete="RESTRICT")
     childs = fields.One2Many('account.account.template', 'parent', 'Children')
     reconcile = fields.Boolean('Reconcile',
             states={
-                'invisible': "kind == 'view'",
+                'invisible': Equal(Eval('kind'), 'view'),
             }, depends=['kind'])
     kind = fields.Selection([
         ('other', 'Other'),
@@ -324,7 +337,7 @@ class AccountTemplate(ModelSQL, ModelView):
         ('view', 'View'),
         ], 'Kind', required=True)
     deferral = fields.Boolean('Deferral', states={
-        'invisible': "kind == 'view'",
+        'invisible': Equal(Eval('kind'), 'view'),
         }, depends=['kind'])
 
     def __init__(self):
@@ -347,7 +360,7 @@ class AccountTemplate(ModelSQL, ModelView):
     def default_deferral(self, cursor, user, context=None):
         return True
 
-    def get_rec_name(self, cursor, user, ids, name, arg, context=None):
+    def get_rec_name(self, cursor, user, ids, name, context=None):
         if not ids:
             return {}
         res = {}
@@ -358,19 +371,13 @@ class AccountTemplate(ModelSQL, ModelView):
                 res[template.id] = template.name
         return res
 
-    def search_rec_name(self, cursor, user, name, args, context=None):
-        args2 = []
-        i = 0
-        while i < len(args):
-            ids = self.search(cursor, user, [
-                ('code', args[i][1], args[i][2]),
-                ], limit=1, context=context)
-            if ids:
-                args2.append(('code', args[i][1], args[i][2]))
-            else:
-                args2.append((self._rec_name, args[i][1], args[i][2]))
-            i += 1
-        return args2
+    def search_rec_name(self, cursor, user, name, clause, context=None):
+        ids = self.search(cursor, user, [
+            ('code',) + clause[1:],
+            ], limit=1, context=context)
+        if ids:
+            return [('code',) + clause[1:]]
+        return [(self._rec_name,) + clause[1:]]
 
     def _get_account_value(self, cursor, user, template, context=None,
             account=None):
@@ -442,6 +449,10 @@ class AccountTemplate(ModelSQL, ModelView):
             new_id = account_obj.create(cursor, user, vals, context=context)
 
             prev_lang = template._context.get('language') or 'en_US'
+            prev_data = {}
+            for field_name, field in template._columns.iteritems():
+                if getattr(field, 'translate', False):
+                    prev_data[field_name] = template[field_name]
             ctx = context.copy()
             for lang in lang_obj.get_translatable_languages(cursor, user,
                     context=context):
@@ -450,9 +461,10 @@ class AccountTemplate(ModelSQL, ModelView):
                 ctx['language'] = lang
                 template.setLang(lang)
                 data = {}
-                for field in template._columns.keys():
-                    if template._columns[field].translate:
-                        data[field] = template[field]
+                for field_name, field in template._columns.iteritems():
+                    if getattr(field, 'translate', False) \
+                            and template[field_name] != prev_data[field_name]:
+                        data[field_name] = template[field_name]
                 if data:
                     account_obj.write(cursor, user, new_id, data, context=ctx)
             template.setLang(prev_lang)
@@ -524,34 +536,37 @@ class Account(ModelSQL, ModelView):
     active = fields.Boolean('Active', select=2)
     company = fields.Many2One('company.company', 'Company', required=True,
             ondelete="RESTRICT")
-    currency = fields.Function('get_currency', type='many2one',
-            relation='currency.currency', string='Currency')
-    currency_digits = fields.Function('get_currency_digits', type='integer',
-            string='Currency Digits')
+    currency = fields.Function(fields.Many2One('currency.currency',
+        'Currency'), 'get_currency')
+    currency_digits = fields.Function(fields.Integer('Currency Digits'),
+            'get_currency_digits')
     second_currency = fields.Many2One('currency.currency', 'Secondary currency',
             help='Force all moves for this account \n' \
                     'to have this secondary currency.', ondelete="RESTRICT")
     type = fields.Many2One('account.account.type', 'Type', ondelete="RESTRICT",
             states={
-                'invisible': "kind == 'view'",
-                'required': "kind != 'view'",
+                'invisible': Equal(Eval('kind'), 'view'),
+                'required': Not(Equal(Eval('kind'), 'view')),
             }, depends=['kind'])
     parent = fields.Many2One('account.account', 'Parent', select=1,
             left="left", right="right", ondelete="RESTRICT")
     left = fields.Integer('Left', required=True, select=1)
     right = fields.Integer('Right', required=True, select=1)
     childs = fields.One2Many('account.account', 'parent', 'Children')
-    balance = fields.Function('get_balance', digits="(16, currency_digits)",
-            string='Balance', depends=['currency_digits'])
-    credit = fields.Function('get_credit_debit', digits="(16, currency_digits)",
-            string='Credit', depends=['currency_digits'])
-    debit = fields.Function('get_credit_debit', digits="(16, currency_digits)",
-            string='Debit', depends=['currency_digits'])
+    balance = fields.Function(fields.Numeric('Balance',
+        digits=(16, Eval('currency_digits', 2)), depends=['currency_digits']),
+        'get_balance')
+    credit = fields.Function(fields.Numeric('Credit',
+        digits=(16, Eval('currency_digits', 2)), depends=['currency_digits']),
+        'get_credit_debit')
+    debit = fields.Function(fields.Numeric('Debit',
+        digits=(16, Eval('currency_digits', 2)), depends=['currency_digits']),
+        'get_credit_debit')
     reconcile = fields.Boolean('Reconcile',
             help='Allow move lines of this account \n' \
                     'to be reconciled.',
             states={
-                'invisible': "kind == 'view'",
+                'invisible': Equal(Eval('kind'), 'view'),
             }, depends=['kind'])
     note = fields.Text('Note')
     kind = fields.Selection([
@@ -563,11 +578,11 @@ class Account(ModelSQL, ModelView):
         ('view', 'View'),
         ], 'Kind', required=True)
     deferral = fields.Boolean('Deferral', states={
-        'invisible': "kind == 'view'",
+        'invisible': Equal(Eval('kind'), 'view'),
         }, depends=['kind'])
     deferrals = fields.One2Many('account.account.deferral', 'account',
             'Deferrals', readonly=True, states={
-                'invisible': "kind == 'view'",
+                'invisible': Equal(Eval('kind'), 'view'),
             }, depends=['kind'])
     template = fields.Many2One('account.account.template', 'Template')
 
@@ -598,7 +613,6 @@ class Account(ModelSQL, ModelView):
         return True
 
     def default_company(self, cursor, user, context=None):
-        company_obj = self.pool.get('company.company')
         if context is None:
             context = {}
         if context.get('company'):
@@ -614,20 +628,20 @@ class Account(ModelSQL, ModelView):
     def default_kind(self, cursor, user, context=None):
         return 'view'
 
-    def get_currency(self, cursor, user, ids, name, arg, context=None):
+    def get_currency(self, cursor, user, ids, name, context=None):
         currency_obj = self.pool.get('currency.currency')
         res = {}
         for account in self.browse(cursor, user, ids, context=context):
             res[account.id] = account.company.currency.id
         return res
 
-    def get_currency_digits(self, cursor, user, ids, name, arg, context=None):
+    def get_currency_digits(self, cursor, user, ids, name, context=None):
         res = {}
         for account in self.browse(cursor, user, ids, context=context):
             res[account.id] = account.company.currency.digits
         return res
 
-    def get_balance(self, cursor, user, ids, name, arg, context=None):
+    def get_balance(self, cursor, user, ids, name, context=None):
         res = {}
         company_obj = self.pool.get('company.company')
         currency_obj = self.pool.get('currency.currency')
@@ -722,8 +736,7 @@ class Account(ModelSQL, ModelView):
                 ctx['fiscalyear'] = fiscalyear.id
                 if 'date' in context:
                     del context['date']
-                res2 = self.get_balance(cursor, user, ids, name, arg,
-                        context=ctx)
+                res2 = self.get_balance(cursor, user, ids, name, context=ctx)
                 for account_id in ids:
                     res[account_id] += res2[account_id]
 
@@ -734,7 +747,7 @@ class Account(ModelSQL, ModelView):
                     res[account_id])
         return res
 
-    def get_credit_debit(self, cursor, user, ids, names, arg, context=None):
+    def get_credit_debit(self, cursor, user, ids, names, context=None):
         '''
         Function to compute debit, credit for account ids.
 
@@ -742,7 +755,6 @@ class Account(ModelSQL, ModelView):
         :param user: the user id
         :param ids: the ids of the account
         :param names: the list of field name to compute
-        :param arg: optional argument
         :param context: the context
         :return: a dictionary with all field names as key and
             a dictionary as value with id as key
@@ -836,7 +848,7 @@ class Account(ModelSQL, ModelView):
                 ctx['fiscalyear'] = fiscalyear.id
                 if 'date' in ctx:
                     del ctx['date']
-                res2 = self.get_credit_debit(cursor, user, ids, names, arg,
+                res2 = self.get_credit_debit(cursor, user, ids, names,
                         context=ctx)
                 for account_id in ids:
                     for name in names:
@@ -850,7 +862,7 @@ class Account(ModelSQL, ModelView):
                         currency, res[name][account_id])
         return res
 
-    def get_rec_name(self, cursor, user, ids, name, arg, context=None):
+    def get_rec_name(self, cursor, user, ids, name, context=None):
         if not ids:
             return {}
         res = {}
@@ -861,19 +873,13 @@ class Account(ModelSQL, ModelView):
                 res[account.id] = account.name
         return res
 
-    def search_rec_name(self, cursor, user, name, args, context=None):
-        args2 = []
-        i = 0
-        while i < len(args):
-            ids = self.search(cursor, user, [
-                ('code', args[i][1], args[i][2]),
-                ], limit=1, context=context)
-            if ids:
-                args2.append(('code', args[i][1], args[i][2]))
-            else:
-                args2.append((self._rec_name, args[i][1], args[i][2]))
-            i += 1
-        return args2
+    def search_rec_name(self, cursor, user, name, clause, context=None):
+        ids = self.search(cursor, user, [
+            ('code',) + clause[1:],
+            ], limit=1, context=context)
+        if ids:
+            return [('code',) + clause[1:]]
+        return [(self._rec_name,) + clause[1:]]
 
     def copy(self, cursor, user, ids, default=None, context=None):
         if default is None:
@@ -953,6 +959,10 @@ class Account(ModelSQL, ModelView):
                 self.write(cursor, user, account.id, vals, context=context)
 
             prev_lang = account._context.get('language') or 'en_US'
+            prev_data = {}
+            for field_name, field in template._columns.iteritems():
+                if getattr(field, 'translate', False):
+                    prev_data[field_name] = template[field_name]
             ctx = context.copy()
             for lang in lang_obj.get_translatable_languages(cursor, user,
                     context=context):
@@ -960,8 +970,11 @@ class Account(ModelSQL, ModelView):
                     continue
                 ctx['language'] = lang
                 account.setLang(lang)
-                data = template_obj._get_account_value(cursor, user,
-                           account.template, context=ctx, account=account)
+                data = {}
+                for field_name, field in template._columns.iteritems():
+                    if getattr(field, 'translate', False) \
+                            and template[field_name] != prev_data[field_name]:
+                        data[field_name] = template[field_name]
                 if data:
                     self.write(cursor, user, account.id, data, context=ctx)
             account.setLang(prev_lang)
@@ -1032,12 +1045,12 @@ class AccountDeferral(ModelSQL, ModelView):
             select=1)
     fiscalyear = fields.Many2One('account.fiscalyear', 'Fiscal Year',
             required=True, select=1)
-    debit = fields.Numeric('Debit', digits="(16, currency_digits)",
+    debit = fields.Numeric('Debit', digits=(16, Eval('currency_digits', 2)),
             depends=['currency_digits'])
-    credit = fields.Numeric('Credit', digits="(16, currency_digits)",
+    credit = fields.Numeric('Credit', digits=(16, Eval('currency_digits', 2)),
             depends=['currency_digits'])
-    currency_digits = fields.Function('get_currency_digits', type='integer',
-            string='Currency Digits')
+    currency_digits = fields.Function(fields.Integer('Currency Digits'),
+            'get_currency_digits')
 
     def __init__(self):
         super(AccountDeferral, self).__init__()
@@ -1049,13 +1062,13 @@ class AccountDeferral(ModelSQL, ModelView):
             'write_deferral': 'You can not modify Account Deferral records',
             })
 
-    def get_currency_digits(self, cursor, user, ids, name, arg, context=None):
+    def get_currency_digits(self, cursor, user, ids, name, context=None):
         res = {}
         for deferral in self.browse(cursor, user, ids, context=context):
             res[deferral.id] = deferral.account.currency_digits
         return res
 
-    def get_rec_name(self, cursor, user, ids, name, arg, context=None):
+    def get_rec_name(self, cursor, user, ids, name, context=None):
         if not ids:
             return {}
         res = {}
@@ -1064,17 +1077,12 @@ class AccountDeferral(ModelSQL, ModelView):
                     deferral.fiscalyear.rec_name
         return res
 
-    def search_rec_name(self, cursor, user, name, args, context=None):
-        args2 = []
-        i = 0
-        while i < len(args):
-            ids = self.search(cursor, user, ['OR',
-                ('account.rec_name', args[i][1], args[i][2]),
-                ('fiscalyear.rec_name', args[i][1], args[i][2]),
-                ], context=context)
-            args2.append(('id', 'in', ids))
-            i += 1
-        return args2
+    def search_rec_name(self, cursor, user, name, clause, context=None):
+        ids = self.search(cursor, user, ['OR',
+            ('account.rec_name',) + clause[1:],
+            ('fiscalyear.rec_name',) + clause[1:],
+            ], context=context)
+        return [('id', 'in', ids)]
 
     def write(self, cursor, user, ids, vals, context=None):
         self.raise_user_error(cursor, 'write_deferral', context=context)
@@ -1122,16 +1130,10 @@ class OpenChartAccount(Wizard):
     def _action_open_chart(self, cursor, user, data, context=None):
         model_data_obj = self.pool.get('ir.model.data')
         act_window_obj = self.pool.get('ir.action.act_window')
-
-        model_data_ids = model_data_obj.search(cursor, user, [
-            ('fs_id', '=', 'act_account_tree2'),
-            ('module', '=', 'account'),
-            ('inherit', '=', False),
-            ], limit=1, context=context)
-        model_data = model_data_obj.browse(cursor, user, model_data_ids[0],
-                context=context)
-        res = act_window_obj.read(cursor, user, model_data.db_id, context=context)
-        res['context'] = str({
+        act_window_id = model_data_obj.get_id(cursor, user, 'account',
+                'act_account_tree2', context=context)
+        res = act_window_obj.read(cursor, user, act_window_id, context=context)
+        res['pyson_context'] = PYSONEncoder().encode({
             'fiscalyear': data['form']['fiscalyear'],
             'posted': data['form']['posted'],
             })
@@ -1147,11 +1149,15 @@ class PrintGeneralLegderInit(ModelView):
     fiscalyear = fields.Many2One('account.fiscalyear', 'Fiscal Year',
             required=True, on_change=['fiscalyear'])
     start_period = fields.Many2One('account.period', 'Start Period',
-            domain=["('fiscalyear', '=', fiscalyear)",
-                "('start_date', '<=', (end_period, 'start_date'))"])
+            domain=[
+                ('fiscalyear', '=', Eval('fiscalyear')),
+                ('start_date', '<=', (Eval('end_period'), 'start_date')),
+            ])
     end_period = fields.Many2One('account.period', 'End Period',
-            domain=["('fiscalyear', '=', fiscalyear)",
-                    "('start_date', '>=', (start_period, 'start_date'))"])
+            domain=[
+                ('fiscalyear', '=', Eval('fiscalyear')),
+                ('start_date', '>=', (Eval('start_period'), 'start_date'))
+            ])
     company = fields.Many2One('company.company', 'Company', required=True)
     posted = fields.Boolean('Posted Move', help='Show only posted move')
     empty_account = fields.Boolean('Empty Account',
@@ -1170,7 +1176,6 @@ class PrintGeneralLegderInit(ModelView):
     def default_company(self, cursor, user, context=None):
         if context is None:
             context = {}
-        company_obj = self.pool.get('company.company')
         if context.get('company'):
             return context['company']
         return False
@@ -1181,7 +1186,7 @@ class PrintGeneralLegderInit(ModelView):
     def default_empty_account(self, cursor, user, context=None):
         return False
 
-    def on_change_fiscalyear(self, cursor, user, ids, vals, context=None):
+    def on_change_fiscalyear(self, cursor, user, vals, context=None):
         return {
             'start_period': False,
             'end_period': False,
@@ -1287,64 +1292,67 @@ class GeneralLegder(Report):
         context['end_period'] = periods[-1]
 
         if not datas['form']['empty_account']:
-            to_remove = []
-            for account_id in account_ids:
-                if not self.get_line_ids(cursor, user, account_id,
-                        end_period_ids, datas['form']['posted'], context):
-                    to_remove.append(account_id)
-            for account_id in to_remove:
-                account_ids.remove(account_id)
+            account_id2lines = self.get_lines(cursor, user, account_ids,
+                    end_period_ids, datas['form']['posted'], context)
+            account_ids = account_id2lines.keys()
+            account_id2lines = None
+
+        account_id2lines = self.lines(cursor, user, account_ids,
+                list(set(end_period_ids).difference(
+                set(start_period_ids))), datas['form']['posted'], context)
 
         context['accounts'] = account_obj.browse(cursor, user, account_ids,
                 context=context)
         context['id2start_account'] = id2start_account
         context['id2end_account'] = id2end_account
         context['digits'] = company.currency.digits
-        context['lines'] = lambda account_id: self.lines(cursor, user,
-                account_id, list(set(end_period_ids).difference(
-                    set(start_period_ids))), datas['form']['posted'], context)
+        context['lines'] = lambda account_id: account_id2lines[account_id]
         context['company'] = company
 
         return super(GeneralLegder, self).parse(cursor, user, report, objects,
                 datas, context)
 
-    def get_line_ids(self, cursor, user, account_id, period_ids, posted,
+    def get_lines(self, cursor, user, account_ids, period_ids, posted,
             context):
         move_line_obj = self.pool.get('account.move.line')
         clause = [
-            ('account', '=', account_id),
+            ('account', 'in', account_ids),
             ('period', 'in', period_ids),
             ('state', '!=', 'draft'),
             ]
         if posted:
             clause.append(('move.state', '=', 'posted'))
-        return move_line_obj.search(cursor, user, clause,
-                context=context)
+        move_ids = move_line_obj.search(cursor, user, clause,
+                order=[], context=context)
+        res = {}
+        for move in move_line_obj.browse(cursor, user, move_ids,
+                context=context):
+            res.setdefault(move.account.id, []).append(move)
+        return res
 
-    def lines(self, cursor, user, account_id, period_ids, posted, context):
+    def lines(self, cursor, user, account_ids, period_ids, posted, context):
         move_line_obj = self.pool.get('account.move.line')
         move_obj = self.pool.get('account.move')
-        res = []
-        move_line_ids = self.get_line_ids(cursor, user, account_id, period_ids,
-                posted, context)
-        move_lines = move_line_obj.browse(cursor, user, move_line_ids,
-                context=context)
-        move_lines.sort(lambda x, y: cmp(x.date, y.date))
+        res = dict((account_id, []) for account_id in account_ids)
+        account_id2lines = self.get_lines(cursor, user, account_ids,
+                period_ids, posted, context)
 
         state_selections = dict(move_obj.fields_get(cursor, user,
                 fields_names=['state'], context=context)['state']['selection'])
 
-        balance = Decimal('0.0')
-        for line in move_lines:
-            balance += line.debit - line.credit
-            res.append({
-                'date': line.date,
-                'debit': line.debit,
-                'credit': line.credit,
-                'balance': balance,
-                'name': line.name,
-                'state': state_selections.get(line.move.state, line.move.state),
-                })
+        for account_id, lines in account_id2lines.iteritems():
+            lines.sort(lambda x, y: cmp(x.date, y.date))
+            balance = Decimal('0.0')
+            for line in lines:
+                balance += line.debit - line.credit
+                res[account_id].append({
+                    'date': line.date,
+                    'debit': line.debit,
+                    'credit': line.credit,
+                    'balance': balance,
+                    'name': line.name,
+                    'state': state_selections.get(line.move.state, line.move.state),
+                    })
         return res
 
 GeneralLegder()
@@ -1358,12 +1366,16 @@ class PrintTrialBalanceInit(ModelView):
             required=True, on_change=['fiscalyear'],
             depends=['start_period', 'end_period'])
     start_period = fields.Many2One('account.period', 'Start Period',
-            domain=["('fiscalyear', '=', fiscalyear)",
-                "('start_date', '<=', (end_period, 'start_date'))"],
+            domain=[
+                ('fiscalyear', '=', Eval('fiscalyear')),
+                ('start_date', '<=', (Eval('end_period'), 'start_date'))
+            ],
             depends=['end_period'])
     end_period = fields.Many2One('account.period', 'End Period',
-            domain=["('fiscalyear', '=', fiscalyear)",
-                "('start_date', '>=', (start_period, 'start_date'))"],
+            domain=[
+                ('fiscalyear', '=', Eval('fiscalyear')),
+                ('start_date', '>=', (Eval('start_period'), 'start_date'))
+            ],
             depends=['start_period'])
     company = fields.Many2One('company.company', 'Company', required=True)
     posted = fields.Boolean('Posted Move', help='Show only posted move')
@@ -1383,7 +1395,6 @@ class PrintTrialBalanceInit(ModelView):
     def default_company(self, cursor, user, context=None):
         if context is None:
             context = {}
-        company_obj = self.pool.get('company.company')
         if context.get('company'):
             return context['company']
         return False
@@ -1394,7 +1405,7 @@ class PrintTrialBalanceInit(ModelView):
     def default_empty_account(self, cursor, user, context=None):
         return False
 
-    def on_change_fiscalyear(self, cursor, user, ids, vals, context=None):
+    def on_change_fiscalyear(self, cursor, user, vals, context=None):
         return {
             'start_period': False,
             'end_period': False,
@@ -1530,7 +1541,6 @@ class OpenBalanceSheetInit(ModelView):
     def default_company(self, cursor, user, context=None):
         if context is None:
             context = {}
-        company_obj = self.pool.get('company.company')
         if context.get('company'):
             return context['company']
         return False
@@ -1582,19 +1592,15 @@ class OpenBalanceSheet(Wizard):
                 break
         lang = lang_obj.browse(cursor, user, lang_ids[0], context=context)
 
-        date = mx.DateTime.strptime(str(datas['form']['date']), '%Y-%m-%d')
-        date = date.strftime(lang.date)
+        date = lang_obj.strftime(datas['form']['date'], lang.code, lang.date)
 
-        model_data_ids = model_data_obj.search(cursor, user, [
-            ('fs_id', '=', 'act_account_balance_sheet_tree'),
-            ('module', '=', 'account'),
-            ('inherit', '=', False),
-            ], limit=1, context=context)
-        model_data = model_data_obj.browse(cursor, user, model_data_ids[0],
-                context=context)
-        res = act_window_obj.read(cursor, user, model_data.db_id, context=context)
-        res['context'] = str({
-            'date': datas['form']['date'],
+        act_window_id = model_data_obj.get_id(cursor, user, 'account',
+                'act_account_balance_sheet_tree', context=context)
+        res = act_window_obj.read(cursor, user, act_window_id, context=context)
+        res['pyson_context'] = PYSONEncoder().encode({
+            'date': Date(datas['form']['date'].year,
+                datas['form']['date'].month,
+                datas['form']['date'].day),
             'posted': datas['form']['posted'],
             'company': datas['form']['company'],
             })
@@ -1612,12 +1618,16 @@ class OpenIncomeStatementInit(ModelView):
             required=True, on_change=['fiscalyear'],
             depends=['start_period', 'end_period'])
     start_period = fields.Many2One('account.period', 'Start Period',
-            domain=["('fiscalyear', '=', fiscalyear)",
-                "('start_date', '<=', (end_period, 'start_date'))"],
+            domain=[
+                ('fiscalyear', '=', Eval('fiscalyear')),
+                ('start_date', '<=', (Eval('end_period'), 'start_date'))
+            ],
             depends=['end_period'])
     end_period = fields.Many2One('account.period', 'End Period',
-            domain=["('fiscalyear', '=', fiscalyear)",
-                "('start_date', '>=', (start_period, 'start_date'))"],
+            domain=[
+                ('fiscalyear', '=', Eval('fiscalyear')),
+                ('start_date', '>=', (Eval('start_period'), 'start_date')),
+            ],
             depends=['start_period'])
     company = fields.Many2One('company.company', 'Company', required=True)
     posted = fields.Boolean('Posted Move', help='Show only posted move')
@@ -1635,7 +1645,6 @@ class OpenIncomeStatementInit(ModelView):
     def default_company(self, cursor, user, context=None):
         if context is None:
             context = {}
-        company_obj = self.pool.get('company.company')
         if context.get('company'):
             return context['company']
         return False
@@ -1643,7 +1652,7 @@ class OpenIncomeStatementInit(ModelView):
     def default_posted(self, cursor, user, context=None):
         return False
 
-    def on_change_fiscalyear(self, cursor, user, ids, vals, context=None):
+    def on_change_fiscalyear(self, cursor, user, vals, context=None):
         return {
             'start_period': False,
             'end_period': False,
@@ -1710,15 +1719,10 @@ class OpenIncomeStatement(Wizard):
             end_period_ids = list(set(end_period_ids).difference(
                 set(start_period_ids)))
 
-        model_data_ids = model_data_obj.search(cursor, user, [
-            ('fs_id', '=', 'act_account_income_statement_tree'),
-            ('module', '=', 'account'),
-            ('inherit', '=', False),
-            ], limit=1, context=context)
-        model_data = model_data_obj.browse(cursor, user, model_data_ids[0],
-                context=context)
-        res = act_window_obj.read(cursor, user, model_data.db_id, context=context)
-        res['context'] = str({
+        act_window_id = model_data_obj.get_id(cursor, user, 'account',
+                'act_account_income_statement_tree', context=context)
+        res = act_window_obj.read(cursor, user, act_window_id, context=context)
+        res['pyson_context'] = PYSONEncoder().encode({
             'periods': end_period_ids,
             'posted': datas['form']['posted'],
             'company': datas['form']['company'],
@@ -1744,6 +1748,13 @@ class CreateChartAccountAccount(ModelView):
     account_template = fields.Many2One('account.account.template',
             'Account Template', required=True, domain=[('parent', '=', False)])
 
+    def default_company(self, cursor, user, context=None):
+        if context is None:
+            context = {}
+        if context.get('company'):
+            return context['company']
+        return False
+
 CreateChartAccountAccount()
 
 
@@ -1754,11 +1765,17 @@ class CreateChartAccountPropertites(ModelView):
     company = fields.Many2One('company.company', 'Company')
     account_receivable = fields.Many2One('account.account',
             'Default Receivable Account',
-            domain=[('kind', '=', 'receivable'), "('company', '=', company)"],
+            domain=[
+                ('kind', '=', 'receivable'),
+                ('company', '=', Eval('company')),
+            ],
             depends=['company'])
     account_payable = fields.Many2One('account.account',
             'Default Payable Account',
-            domain=[('kind', '=', 'payable'), "('company', '=', company)"],
+            domain=[
+                ('kind', '=', 'payable'),
+                ('company', '=', Eval('company')),
+            ],
             depends=['company'])
 
 CreateChartAccountPropertites()
@@ -1905,8 +1922,8 @@ class CreateChartAccount(Wizard):
             ('res', '=', False),
             ('company', '=', datas['form']['company']),
             ], context=context)
-        property_obj.delete(cursor, user, property_ids, context=context)
-        property_obj.create(cursor, user, {
+        property_obj.delete(cursor, 0, property_ids, context=context)
+        property_obj.create(cursor, 0, {
             'name': 'account_receivable',
             'field': account_receivable_field_id,
             'value': 'account.account,' + \
@@ -1923,8 +1940,8 @@ class CreateChartAccount(Wizard):
             ('res', '=', False),
             ('company', '=', datas['form']['company']),
             ], context=context)
-        property_obj.delete(cursor, user, property_ids, context=context)
-        property_obj.create(cursor, user, {
+        property_obj.delete(cursor, 0, property_ids, context=context)
+        property_obj.create(cursor, 0, {
             'name': 'account_payable',
             'field': account_payable_field_id,
             'value': 'account.account,' + \
@@ -2143,7 +2160,6 @@ class OpenThirdPartyBalanceInit(ModelView):
     def default_company(self, cursor, user, context=None):
         if context is None:
             context = {}
-        company_obj = self.pool.get('company.company')
         if context.get('company'):
             return context['company']
         return False
@@ -2286,7 +2302,6 @@ class OpenAgedBalanceInit(ModelView):
     def default_company(self, cursor, user, context=None):
         if context is None:
             context = {}
-        company_obj = self.pool.get('company.company')
         if context.get('company'):
             return context['company']
         return False
