@@ -3,10 +3,14 @@
 'Fiscal Year'
 from trytond.model import ModelView, ModelSQL, fields
 from trytond.wizard import Wizard
-import mx.DateTime
+from trytond.tools import datetime_strftime
+from trytond.pyson import Equal, Eval, If, In, Get
+import datetime
+import time
+from dateutil.relativedelta import relativedelta
 
 STATES = {
-    'readonly': "state == 'close'",
+    'readonly': Equal(Eval('state'), 'close'),
 }
 DEPENDS = ['state']
 
@@ -29,15 +33,21 @@ class FiscalYear(ModelSQL, ModelView):
         ('close', 'Close'),
         ], 'State', readonly=True, required=True)
     post_move_sequence = fields.Many2One('ir.sequence', 'Post Move Sequence',
-            required=True, domain=["('code', '=', 'account.move')",
+            required=True, domain=[('code', '=', 'account.move'),
                 ['OR',
-                    "('company', '=', company)",
-                    "('company', '=', False)"]],
-                    context="{'code': 'account.move', 'company': company}",
+                    ('company', '=', Eval('company')),
+                    ('company', '=', False)
+                ]],
+            context={
+                'code': 'account.move',
+                'company': Eval('company'),
+            },
             depends=['company'])
     company = fields.Many2One('company.company', 'Company', required=True,
-            domain=["('id', 'company' in context and '=' or '!=', " \
-                    "context.get('company', 0))"])
+            domain=[
+                ('id', If(In('company', Eval('context', {})), '=', '!='),
+                    Get(Eval('context', {}), 'company', 0)),
+            ])
 
     def __init__(self):
         super(FiscalYear, self).__init__()
@@ -72,7 +82,6 @@ class FiscalYear(ModelSQL, ModelView):
         return 'open'
 
     def default_company(self, cursor, user, context=None):
-        company_obj = self.pool.get('company.company')
         if context is None:
             context = {}
         if context.get('company'):
@@ -139,31 +148,25 @@ class FiscalYear(ModelSQL, ModelView):
         '''
         period_obj = self.pool.get('account.period')
         for fiscalyear in self.browse(cursor, user, ids, context=context):
-            end_date = mx.DateTime.strptime(str(fiscalyear.end_date),
-                    '%Y-%m-%d')
-            period_start_date = mx.DateTime.strptime(str(fiscalyear.start_date),
-                    '%Y-%m-%d')
-            while period_start_date < end_date:
+            period_start_date = fiscalyear.start_date
+            while period_start_date < fiscalyear.end_date:
                 period_end_date = period_start_date + \
-                        mx.DateTime.RelativeDateTime(months=interval)
-                period_end_date = mx.DateTime.DateTime(period_end_date.year,
-                        period_end_date.month, 1) - \
-                        mx.DateTime.RelativeDateTime(days=1)
-                if period_end_date > end_date:
-                    period_end_date = end_date
-                name = period_start_date.strftime('%Y-%m')
-                if name != period_end_date.strftime('%Y-%m'):
-                    name += ' - ' + period_end_date.strftime('%Y-%m')
+                        relativedelta(months=interval - 1) + \
+                        relativedelta(day=31)
+                if period_end_date > fiscalyear.end_date:
+                    period_end_date = fiscalyear.end_date
+                name = datetime_strftime(period_start_date, '%Y-%m')
+                if name != datetime_strftime(period_end_date, '%Y-%m'):
+                    name += ' - ' + datetime_strftime(period_end_date, '%Y-%m')
                 period_obj.create(cursor, user, {
                     'name': name,
-                    'start_date': period_start_date.strftime('%Y-%m-%d'),
-                    'end_date': period_end_date.strftime('%Y-%m-%d'),
+                    'start_date': period_start_date,
+                    'end_date': period_end_date,
                     'fiscalyear': fiscalyear.id,
                     'post_move_sequence': fiscalyear.post_move_sequence.id,
                     'type': 'standard',
                     }, context=context)
-                period_start_date = period_end_date + \
-                        mx.DateTime.RelativeDateTime(days=1)
+                period_start_date = period_end_date + relativedelta(days=1)
         return True
 
     def create_period_3(self, cursor, user, ids, context=None):
@@ -316,9 +319,8 @@ class CloseFiscalYearInit(ModelView):
     _name = 'account.fiscalyear.close_fiscalyear.init'
     _description = __doc__
     close_fiscalyear = fields.Many2One('account.fiscalyear',
-            'Fiscal Year to close',
-            required=True,
-            domain=["('state', '!=', 'close')"])
+            'Fiscal Year to close', required=True,
+            domain=[('state', '!=', 'close')])
 
 CloseFiscalYearInit()
 
