@@ -4,9 +4,10 @@
 from trytond.model import ModelView, ModelSQL, fields
 from trytond.wizard import Wizard
 from trytond.backend import TableHandler
+from trytond.pyson import Equal, Eval, Not, Get, Or, And, Bool
 
 STATES = {
-    'readonly': "state == 'close'",
+    'readonly': Equal(Eval('state'), 'close'),
 }
 DEPENDS = ['state']
 
@@ -84,28 +85,34 @@ class Journal(ModelSQL, ModelView):
     view = fields.Many2One('account.journal.view', 'View')
     centralised = fields.Boolean('Centralised counterpart')
     update_posted = fields.Boolean('Allow cancelling moves')
-    sequence = fields.Property(type='many2one', relation='ir.sequence',
-            string='Sequence', required=True,
-            domain=[('code', '=', 'account.journal')],
-            context={'code': 'account.journal'})
-    credit_account = fields.Property(type='many2one',
-            relation='account.account', string='Default Credit Account',
-            domain=[('kind', '!=', 'view'),
-                "('company', '=', context.get('company'))"],
-            states={
-                'required': "centralised or " \
-                        "(type == 'cash' and context.get('company'))",
-                'invisible': "not context.get('company')",
-            }, depends=['type', 'centralised'])
-    debit_account = fields.Property(type='many2one',
-            relation='account.account', string='Default Debit Account',
-            domain=[('kind', '!=', 'view'),
-                "('company', '=', context.get('company'))"],
-            states={
-                'required': "centralised or " \
-                        "(type == 'cash' and context.get('company'))",
-                'invisible': "not context.get('company')",
-            }, depends=['type', 'centralised'])
+    sequence = fields.Property(fields.Many2One('ir.sequence', 'Sequence',
+        domain=[('code', '=', 'account.journal')],
+        context={'code': 'account.journal'},
+        states={
+            'required': Bool(Get(Eval('context', {}), 'company', 0)),
+        }))
+    credit_account = fields.Property(fields.Many2One('account.account',
+        'Default Credit Account', domain=[
+            ('kind', '!=', 'view'),
+            ('company', '=', Get(Eval('context', {}), 'company', 0)),
+        ],
+        states={
+            'required': Or(Bool(Eval('centralised')),
+                And(Equal(Eval('type'), 'cash'),
+                    Bool(Get(Eval('context', {}), 'company', 0)))),
+            'invisible': Not(Bool(Get(Eval('context', {}), 'company', 0))),
+        }, depends=['type', 'centralised']))
+    debit_account = fields.Property(fields.Many2One('account.account',
+        'Default Debit Account', domain=[
+            ('kind', '!=', 'view'),
+            ('company', '=', Get(Eval('context', {}), 'company', 0)),
+        ],
+        states={
+            'required': Or(Bool(Eval('centralised')),
+                And(Equal(Eval('type'), 'cash'),
+                    Bool(Get(Eval('context', {}), 'company', 0)))),
+            'invisible': Not(Bool(Get(Eval('context', {}), 'company', 0))),
+        }, depends=['type', 'centralised']))
 
     def __init__(self):
         super(Journal, self).__init__()
@@ -143,19 +150,13 @@ class Journal(ModelSQL, ModelView):
         types = type_obj.browse(cursor, user, type_ids, context=context)
         return [(x.code, x.name) for x in types]
 
-    def search_rec_name(self, cursor, user, name, args, context=None):
-        args2 = []
-        i = 0
-        while i < len(args):
-            ids = self.search(cursor, user, [
-                ('code', args[i][1], args[i][2]),
-                ], limit=1, context=context)
-            if ids:
-                args2.append(('code', args[i][1], args[i][2]))
-            else:
-                args2.append((self._rec_name, args[i][1], args[i][2]))
-            i += 1
-        return args2
+    def search_rec_name(self, cursor, user, name, clause, context=None):
+        ids = self.search(cursor, user, [
+            ('code',) + clause[1:],
+            ], limit=1, order=[], context=context)
+        if ids:
+            return [('code',) + clause[1:]]
+        return [(self._rec_name,) + clause[1:]]
 
 Journal()
 
@@ -170,7 +171,7 @@ class Period(ModelSQL, ModelView):
             ondelete='CASCADE', states=STATES, depends=DEPENDS)
     period = fields.Many2One('account.period', 'Period', required=True,
             ondelete='CASCADE', states=STATES, depends=DEPENDS)
-    icon = fields.Function('get_icon', string='Icon', type='char')
+    icon = fields.Function(fields.Char('Icon'), 'get_icon')
     active = fields.Boolean('Active', select=2, states=STATES, depends=DEPENDS)
     state = fields.Selection([
         ('open', 'Open'),
@@ -199,7 +200,7 @@ class Period(ModelSQL, ModelView):
     def default_state(self, cursor, user, context=None):
         return 'open'
 
-    def get_icon(self, cursor, user, ids, name, arg, context=None):
+    def get_icon(self, cursor, user, ids, name, context=None):
         res = {}
         for period in self.browse(cursor, user, ids, context=context):
             res[period.id] = _ICONS.get(period.state, '')
@@ -242,8 +243,7 @@ class Period(ModelSQL, ModelView):
 
     def delete(self, cursor, user, ids, context=None):
         self._check(cursor, user, ids, context=context)
-        return super(Period, self).delete(cursor, user, ids, vals,
-                context=context)
+        return super(Period, self).delete(cursor, user, ids, context=context)
 
     def close(self, cursor, user, ids, context=None):
         '''
