@@ -10,8 +10,7 @@ if os.path.isdir(DIR):
 
 import unittest
 import trytond.tests.test_tryton
-import trytond.modules.company.tests
-from trytond.tests.test_tryton import RPCProxy, CONTEXT, SOCK, test_view
+from trytond.tests.test_tryton import test_view
 
 
 class AccountTestCase(unittest.TestCase):
@@ -29,12 +28,10 @@ class AccountTestCase(unittest.TestCase):
         self.assertRaises(Exception, test_view('account'))
 
 def suite():
-    return unittest.TestLoader().loadTestsFromTestCase(AccountTestCase)
+    suite = trytond.tests.test_tryton.suite()
+    suite.addTests(unittest.TestLoader().loadTestsFromTestCase(
+        AccountTestCase))
+    return suite
 
 if __name__ == '__main__':
-    suiteTrytond = trytond.tests.test_tryton.suite()
-    suiteCompany = trytond.modules.company.tests.suite()
-    suiteAccount = suite()
-    alltests = unittest.TestSuite([suiteTrytond, suiteCompany, suiteAccount])
-    unittest.TextTestRunner(verbosity=2).run(alltests)
-    SOCK.disconnect()
+    unittest.TextTestRunner(verbosity=2).run(suite())
